@@ -12,6 +12,8 @@ import numpy as np
 from .colors import ContinousColorScales, DiscreteColorScales
 import plotly.colors as pc
 
+from funcnodes import NoValue
+
 
 @fn.NodeDecorator(
     "plotly.express.scatter",
@@ -34,7 +36,7 @@ import plotly.colors as pc
                         "facet_row",
                         "facet_col",
                     ],
-                    lambda x: list(iter(x)),
+                    lambda x: [NoValue] + list(iter(x)),
                 )
             }
         },
@@ -50,12 +52,15 @@ def scatter(
     facet_row: Optional[str] = None,
     facet_col: Optional[str] = None,
     facet_col_wrap: Optional[int] = None,
+    node: Optional[fn.Node] = None,
 ) -> go.Figure:
     """
     Create a scatter plot.
     """
+
     return px.scatter(
         data,
+        title=node.name if node else None,
         x=x,
         y=y,
         color=color,
@@ -86,7 +91,7 @@ def scatter(
                         "facet_row",
                         "facet_col",
                     ],
-                    lambda x: list(iter(x)),
+                    lambda x: [NoValue] + list(iter(x)),
                 )
             }
         },
@@ -100,12 +105,14 @@ def line(
     facet_row: Optional[str] = None,
     facet_col: Optional[str] = None,
     facet_col_wrap: Optional[int] = None,
+    node: Optional[fn.Node] = None,
 ) -> go.Figure:
     """
     Create a line plot.
     """
     return px.line(
         data,
+        title=node.name if node else None,
         x=x,
         y=y,
         color=color,
@@ -134,7 +141,7 @@ def line(
                         "facet_row",
                         "facet_col",
                     ],
-                    lambda x: list(iter(x)),
+                    lambda x: [NoValue] + list(iter(x)),
                 )
             }
         },
@@ -148,12 +155,14 @@ def bar(
     facet_row: Optional[str] = None,
     facet_col: Optional[str] = None,
     facet_col_wrap: Optional[int] = None,
+    node: Optional[fn.Node] = None,
 ) -> go.Figure:
     """
     Create a bar plot.
     """
     return px.bar(
         data,
+        title=node.name if node else None,
         x=x,
         y=y,
         color=color,
@@ -182,7 +191,7 @@ def bar(
                         "facet_row",
                         "facet_col",
                     ],
-                    lambda x: list(iter(x)),
+                    lambda x: [NoValue] + list(iter(x)),
                 )
             }
         },
@@ -196,12 +205,14 @@ def area(
     facet_row: Optional[str] = None,
     facet_col: Optional[str] = None,
     facet_col_wrap: Optional[int] = None,
+    node: Optional[fn.Node] = None,
 ) -> go.Figure:
     """
     Create an area plot.
     """
     return px.area(
         data,
+        title=node.name if node else None,
         x=x,
         y=y,
         color=color,
@@ -230,7 +241,7 @@ def area(
                         "facet_row",
                         "facet_col",
                     ],
-                    lambda x: list(iter(x)),
+                    lambda x: [NoValue] + list(iter(x)),
                 )
             }
         },
@@ -244,12 +255,14 @@ def funnel(
     facet_row: Optional[str] = None,
     facet_col: Optional[str] = None,
     facet_col_wrap: Optional[int] = None,
+    node: Optional[fn.Node] = None,
 ) -> go.Figure:
     """
     Create a funnel plot.
     """
     return px.funnel(
         data,
+        title=node.name if node else None,
         x=x,
         y=y,
         color=color,
@@ -279,7 +292,7 @@ def funnel(
                         "facet_row",
                         "facet_col",
                     ],
-                    lambda x: list(iter(x)),
+                    lambda x: [NoValue] + list(iter(x)),
                 )
             }
         },
@@ -294,12 +307,14 @@ def timeline(
     facet_row: Optional[str] = None,
     facet_col: Optional[str] = None,
     facet_col_wrap: Optional[int] = None,
+    node: Optional[fn.Node] = None,
 ) -> go.Figure:
     """
     Create a timeline plot. The x_start and x_end columns should be datetime objects.
     """
     return px.timeline(
         data,
+        title=node.name if node else None,
         x_start=x_start,
         x_end=x_end,
         y=y,
@@ -327,7 +342,7 @@ def timeline(
                         "values",
                         "color",
                     ],
-                    lambda x: list(iter(x)),
+                    lambda x: [NoValue] + list(iter(x)),
                 )
             }
         },
@@ -335,16 +350,18 @@ def timeline(
 )
 def pie(
     data: pd.DataFrame,
-    names: str,
-    values: str,
+    names: Optional[str] = None,
+    values: Optional[str] = None,
     color: Optional[str] = None,
     hole: float = 0,
+    node: Optional[fn.Node] = None,
 ) -> go.Figure:
     """
     Create a pie chart.
     """
     return px.pie(
         data,
+        title=node.name if node else None,
         names=names,
         values=values,
         color=color,
@@ -370,7 +387,7 @@ def pie(
                         "parents",
                         "color",
                     ],
-                    lambda x: list(iter(x)),
+                    lambda x: [NoValue] + list(iter(x)),
                 )
             }
         },
@@ -382,12 +399,14 @@ def sunburst(
     values: str,
     parents: str,
     color: Optional[str] = None,
+    node: Optional[fn.Node] = None,
 ) -> go.Figure:
     """
     Create a sunburst chart.
     """
     fig = px.sunburst(
         data,
+        title=node.name if node else None,
         names=names,
         values=values,
         parents=parents,
@@ -415,7 +434,7 @@ def sunburst(
                         "parents",
                         "color",
                     ],
-                    lambda x: list(iter(x)),
+                    lambda x: [NoValue] + list(iter(x)),
                 )
             }
         },
@@ -427,6 +446,7 @@ def treemap(
     values: str,
     parents: str,
     color: Optional[str] = None,
+    node: Optional[fn.Node] = None,
 ) -> go.Figure:
     """
     Create a treemap.
@@ -434,6 +454,7 @@ def treemap(
 
     fig = px.treemap(
         data,
+        title=node.name if node else None,
         names=names,
         values=values,
         parents=parents,
@@ -461,7 +482,7 @@ def treemap(
                         "parents",
                         "color",
                     ],
-                    lambda x: list(iter(x)),
+                    lambda x: [NoValue] + list(iter(x)),
                 )
             }
         },
@@ -473,12 +494,14 @@ def icicle(
     values: str,
     parents: str,
     color: Optional[str] = None,
+    node: Optional[fn.Node] = None,
 ) -> go.Figure:
     """
     Create an icicle plot.
     """
     fig = px.icicle(
         data,
+        title=node.name if node else None,
         names=names,
         values=values,
         parents=parents,
@@ -505,7 +528,7 @@ def icicle(
                         "values",
                         "color",
                     ],
-                    lambda x: list(iter(x)),
+                    lambda x: [NoValue] + list(iter(x)),
                 )
             }
         },
@@ -516,12 +539,14 @@ def funnel_area(
     names: str,
     values: str,
     color: Optional[str] = None,
+    node: Optional[fn.Node] = None,
 ) -> go.Figure:
     """
     Create a funnel area plot.
     """
     return px.funnel_area(
         data,
+        title=node.name if node else None,
         names=names,
         values=values,
         color=color,
@@ -547,7 +572,7 @@ def funnel_area(
                         "facet_row",
                         "facet_col",
                     ],
-                    lambda x: list(iter(x)),
+                    lambda x: [NoValue] + list(iter(x)),
                 )
             }
         },
@@ -572,6 +597,7 @@ def histogram(
     facet_row: Optional[str] = None,
     facet_col: Optional[str] = None,
     facet_col_wrap: Optional[int] = None,
+    node: Optional[fn.Node] = None,
 ) -> go.Figure:
     """
     Create a histogram.
@@ -580,6 +606,7 @@ def histogram(
         marginal = None
     return px.histogram(
         data,
+        title=node.name if node else None,
         x=x,
         color=color,
         pattern_shape=pattern_shape,
@@ -610,7 +637,7 @@ def histogram(
                         "facet_row",
                         "facet_col",
                     ],
-                    lambda x: list(iter(x)),
+                    lambda x: [NoValue] + list(iter(x)),
                 )
             }
         },
@@ -624,12 +651,14 @@ def box(
     facet_row: Optional[str] = None,
     facet_col: Optional[str] = None,
     facet_col_wrap: Optional[int] = None,
+    node: Optional[fn.Node] = None,
 ) -> go.Figure:
     """
     Create a box plot.
     """
     return px.box(
         data,
+        title=node.name if node else None,
         x=x,
         y=y,
         color=color,
@@ -658,7 +687,7 @@ def box(
                         "facet_row",
                         "facet_col",
                     ],
-                    lambda x: list(iter(x)),
+                    lambda x: [NoValue] + list(iter(x)),
                 )
             }
         },
@@ -672,12 +701,14 @@ def violin(
     facet_row: Optional[str] = None,
     facet_col: Optional[str] = None,
     facet_col_wrap: Optional[int] = None,
+    node: Optional[fn.Node] = None,
 ) -> go.Figure:
     """
     Create a violin plot.
     """
     return px.violin(
         data,
+        title=node.name if node else None,
         x=x,
         y=y,
         color=color,
@@ -706,7 +737,7 @@ def violin(
                         "facet_row",
                         "facet_col",
                     ],
-                    lambda x: list(iter(x)),
+                    lambda x: [NoValue] + list(iter(x)),
                 )
             }
         },
@@ -720,12 +751,14 @@ def strip(
     facet_row: Optional[str] = None,
     facet_col: Optional[str] = None,
     facet_col_wrap: Optional[int] = None,
+    node: Optional[fn.Node] = None,
 ) -> go.Figure:
     """
     Create a strip plot.
     """
     return px.strip(
         data,
+        title=node.name if node else None,
         x=x,
         y=y,
         color=color,
@@ -754,7 +787,7 @@ def strip(
                         "facet_row",
                         "facet_col",
                     ],
-                    lambda x: list(iter(x)),
+                    lambda x: [NoValue] + list(iter(x)),
                 )
             }
         },
@@ -767,12 +800,14 @@ def ecdf(
     facet_row: Optional[str] = None,
     facet_col: Optional[str] = None,
     facet_col_wrap: Optional[int] = None,
+    node: Optional[fn.Node] = None,
 ) -> go.Figure:
     """
     Create an ECDF plot.
     """
     return px.ecdf(
         data,
+        title=node.name if node else None,
         x=x,
         color=color,
         facet_row=facet_row,
@@ -800,7 +835,7 @@ def ecdf(
                         "facet_row",
                         "facet_col",
                     ],
-                    lambda x: list(iter(x)),
+                    lambda x: [NoValue] + list(iter(x)),
                 )
             }
         },
@@ -816,6 +851,9 @@ def density_heatmap(
     facet_row: Optional[str] = None,
     facet_col: Optional[str] = None,
     facet_col_wrap: Optional[int] = None,
+    nbinsx: Optional[int] = None,
+    nbinsy: Optional[int] = None,
+    node: Optional[fn.Node] = None,
 ) -> go.Figure:
     """
     Create a density heatmap.
@@ -824,6 +862,7 @@ def density_heatmap(
         color_continuous_scale = ContinousColorScales.v(color_continuous_scale)
     return px.density_heatmap(
         data,
+        title=node.name if node else None,
         x=x,
         y=y,
         z=z,
@@ -832,6 +871,8 @@ def density_heatmap(
         facet_row=facet_row,
         facet_col=facet_col,
         facet_col_wrap=facet_col_wrap,
+        nbinsx=nbinsx,
+        nbinsy=nbinsy,
     )
 
 
@@ -854,7 +895,7 @@ def density_heatmap(
                         "facet_row",
                         "facet_col",
                     ],
-                    lambda x: list(iter(x)),
+                    lambda x: [NoValue] + list(iter(x)),
                 )
             }
         },
@@ -870,6 +911,7 @@ def density_contour(
     facet_row: Optional[str] = None,
     facet_col: Optional[str] = None,
     facet_col_wrap: Optional[int] = None,
+    node: Optional[fn.Node] = None,
 ) -> go.Figure:
     """
     Create a density contour plot.
@@ -878,6 +920,7 @@ def density_contour(
         color_discrete_sequence = DiscreteColorScales.v(color_discrete_sequence)
     return px.density_contour(
         data,
+        title=node.name if node else None,
         x=x,
         y=y,
         z=z,
@@ -905,6 +948,7 @@ def imshow(
     value_text: bool = False,
     x: Optional[np.ndarray] = None,
     y: Optional[np.ndarray] = None,
+    node: Optional[fn.Node] = None,
 ) -> go.Figure:
     """
     Create an image plot.
@@ -918,6 +962,7 @@ def imshow(
 
     return px.imshow(
         data,
+        title=node.name if node else None,
         color_continuous_scale=color_continuous_scale,
         color_continuous_midpoint=scale_midpoint,
         text_auto=value_text,
@@ -945,7 +990,7 @@ def imshow(
                         "color",
                         "symbol",
                     ],
-                    lambda x: list(iter(x)),
+                    lambda x: [NoValue] + list(iter(x)),
                 )
             }
         },
@@ -958,12 +1003,14 @@ def scatter_3d(
     z: str,
     color: Optional[str] = None,
     symbol: Optional[str] = None,
+    node: Optional[fn.Node] = None,
 ) -> go.Figure:
     """
     Create a 3D scatter plot.
     """
     return px.scatter_3d(
         data,
+        title=node.name if node else None,
         x=x,
         y=y,
         z=z,
@@ -990,7 +1037,7 @@ def scatter_3d(
                         "z",
                         "color",
                     ],
-                    lambda x: list(iter(x)),
+                    lambda x: [NoValue] + list(iter(x)),
                 )
             }
         },
@@ -1002,12 +1049,14 @@ def line_3d(
     y: str,
     z: str,
     color: Optional[str] = None,
+    node: Optional[fn.Node] = None,
 ) -> go.Figure:
     """
     Create a 3D line plot.
     """
     return px.line_3d(
         data,
+        title=node.name if node else None,
         x=x,
         y=y,
         z=z,
@@ -1030,7 +1079,7 @@ def line_3d(
                     [
                         "color",
                     ],
-                    lambda x: list(iter(x)),
+                    lambda x: [NoValue] + list(iter(x)),
                 )
             }
         },
@@ -1040,6 +1089,7 @@ def scatter_matrix(
     data: pd.DataFrame,
     dimensions: Optional[str] = None,
     color: Optional[str] = None,
+    node: Optional[fn.Node] = None,
 ) -> go.Figure:
     """
     Create a scatter matrix.
@@ -1055,6 +1105,7 @@ def scatter_matrix(
 
     return px.scatter_matrix(
         data,
+        title=node.name if node else None,
         dimensions=dimensions_list,
         color=color,
     )
@@ -1075,7 +1126,7 @@ def scatter_matrix(
                     [
                         "color",
                     ],
-                    lambda x: list(iter(x)),
+                    lambda x: [NoValue] + list(iter(x)),
                 )
             }
         },
@@ -1087,6 +1138,7 @@ def parallel_coordinates(
     dimensions: Optional[str] = None,
     color_continuous_scale: Optional[ContinousColorScales] = None,
     color_continuous_midpoint: Optional[float] = None,
+    node: Optional[fn.Node] = None,
 ) -> go.Figure:
     """
     Create a parallel coordinates plot.
@@ -1101,6 +1153,7 @@ def parallel_coordinates(
 
     return px.parallel_coordinates(
         data,
+        title=node.name if node else None,
         color=color,
         dimensions=dimensions_list,
         color_continuous_scale=color_continuous_scale,
@@ -1123,7 +1176,7 @@ def parallel_coordinates(
                     [
                         "color",
                     ],
-                    lambda x: list(iter(x)),
+                    lambda x: [NoValue] + list(iter(x)),
                 )
             }
         },
@@ -1135,6 +1188,7 @@ def parallel_categories(
     color: Optional[str] = None,
     color_continuous_scale: Optional[ContinousColorScales] = None,
     color_continuous_midpoint: Optional[float] = None,
+    node: Optional[fn.Node] = None,
 ) -> go.Figure:
     """
     Create a parallel categories plot.
@@ -1149,6 +1203,7 @@ def parallel_categories(
 
     return px.parallel_categories(
         data,
+        title=node.name if node else None,
         dimensions=dimensions_list,
         color=color,
         color_continuous_scale=color_continuous_scale,
@@ -1174,7 +1229,7 @@ def parallel_categories(
                         "symbol",
                         "color",
                     ],
-                    lambda x: list(iter(x)),
+                    lambda x: [NoValue] + list(iter(x)),
                 )
             }
         },
@@ -1186,12 +1241,14 @@ def scatter_polar(
     theta: str,
     color: Optional[str] = None,
     symbol: Optional[str] = None,
+    node: Optional[fn.Node] = None,
 ) -> go.Figure:
     """
     Create a polar scatter plot.
     """
     return px.scatter_polar(
         data,
+        title=node.name if node else None,
         r=r,
         theta=theta,
         color=color,
@@ -1216,7 +1273,7 @@ def scatter_polar(
                         "theta",
                         "color",
                     ],
-                    lambda x: list(iter(x)),
+                    lambda x: [NoValue] + list(iter(x)),
                 )
             }
         },
@@ -1227,12 +1284,14 @@ def line_polar(
     r: str,
     theta: str,
     color: Optional[str] = None,
+    node: Optional[fn.Node] = None,
 ) -> go.Figure:
     """
     Create a polar line plot.
     """
     return px.line_polar(
         data,
+        title=node.name if node else None,
         r=r,
         theta=theta,
         color=color,
@@ -1256,7 +1315,7 @@ def line_polar(
                         "theta",
                         "color",
                     ],
-                    lambda x: list(iter(x)),
+                    lambda x: [NoValue] + list(iter(x)),
                 )
             }
         },
@@ -1268,6 +1327,7 @@ def bar_polar(
     theta: str,
     color: Optional[str] = None,
     color_discrete_sequence: Optional[DiscreteColorScales] = None,
+    node: Optional[fn.Node] = None,
 ) -> go.Figure:
     """
     Create a polar bar plot.
@@ -1276,6 +1336,7 @@ def bar_polar(
         color_discrete_sequence = ContinousColorScales.v(color_discrete_sequence)
     return px.bar_polar(
         data,
+        title=node.name if node else None,
         r=r,
         theta=theta,
         color=color,
@@ -1302,7 +1363,7 @@ def bar_polar(
                         "symbol",
                         "color",
                     ],
-                    lambda x: list(iter(x)),
+                    lambda x: [NoValue] + list(iter(x)),
                 )
             }
         },
@@ -1315,12 +1376,14 @@ def scatter_ternary(
     c: str,
     color: Optional[str] = None,
     symbol: Optional[str] = None,
+    node: Optional[fn.Node] = None,
 ) -> go.Figure:
     """
     Create a ternary scatter plot.
     """
     return px.scatter_ternary(
         data,
+        title=node.name if node else None,
         a=a,
         b=b,
         c=c,
@@ -1347,7 +1410,7 @@ def scatter_ternary(
                         "c",
                         "color",
                     ],
-                    lambda x: list(iter(x)),
+                    lambda x: [NoValue] + list(iter(x)),
                 )
             }
         },
@@ -1359,12 +1422,14 @@ def line_ternary(
     b: str,
     c: str,
     color: Optional[str] = None,
+    node: Optional[fn.Node] = None,
 ) -> go.Figure:
     """
     Create a ternary line plot.
     """
     return px.line_ternary(
         data,
+        title=node.name if node else None,
         a=a,
         b=b,
         c=c,
@@ -1387,7 +1452,7 @@ def line_ternary(
                     [
                         "x",
                     ],
-                    lambda x: list(iter(x)),
+                    lambda x: ["index"] + list(iter(x)),
                 )
             }
         },
@@ -1398,6 +1463,7 @@ def plot_multidata(
     x: str,
     mode: Literal["lines", "markers", "lines+markers"] = "lines",
     stack: bool = False,
+    node: Optional[fn.Node] = None,
 ) -> go.Figure:
     """
     Create a plot from a dictionary where one key is used as the x-axis and the remaining keys are used as
@@ -1420,11 +1486,17 @@ def plot_multidata(
             and value.ndim < 2
         )
 
-    if x not in data:
-        raise ValueError(f"The specified x '{x}' is not in the dictionary.")
+    if not isinstance(data, pd.DataFrame):
+        data = pd.DataFrame(data)
 
-    # Extract the x values
-    x_values = data[x]
+    if x == "index":
+        x_values = data.index
+    else:
+        if x not in data:
+            raise ValueError(f"The specified x '{x}' is not in the data.")
+
+        # Extract the x values
+        x_values = data[x]
 
     # Create figure
     fig = go.Figure()
@@ -1467,6 +1539,7 @@ def plot_multidata(
 
         # Update layout with axes configurations
         fig.update_layout(
+            title=node.name if node else None,
             xaxis=dict(
                 autorange=True,
                 title=x,
